@@ -102,7 +102,12 @@ class SlackMode:
 
     # We're dealing with a webhook
     # Our token looks like: T1JJ3T3L2/A1BRTD4JD/TIiajkdnlazkcOXrIdevi7
-    WEBHOOK = "webhook"
+    WEBHOOK = "hook"
+
+    # Government Webhook
+    # Our token still looks like: T1JJ3T3L2/A1BRTD4JD/TIiajkdnlazkcOXrIdevi7
+    # however we have a different URL we post to
+    WEBHOOK_GOV = "gov-hook"
 
     # We're dealing with a bot (using the OAuth Access Token)
     # Our token looks like: xoxp-1234-1234-1234-abc124 or
@@ -113,6 +118,7 @@ class SlackMode:
 # Define our Slack Modes
 SLACK_MODES = (
     SlackMode.WEBHOOK,
+    SlackMode.WEBHOOK_GOV,
     SlackMode.BOT,
 )
 
@@ -140,8 +146,9 @@ class NotifySlack(NotifyBase):
     attachment_support = True
 
     # The maximum targets to include when doing batch transfers
-    # Slack Webhook URL
+    # Slack Webhook URLs
     webhook_url = "https://hooks.slack.com/services"
+    webhook_gov_url = "https://hooks.slack-gov.com/services"
 
     # Slack API URL (used with Bots)
     api_url = "https://slack.com/api/{}"
@@ -271,6 +278,18 @@ class NotifySlack(NotifyBase):
             "to": {
                 "alias_of": "targets",
             },
+            "timestamp": {
+                "name": _("Include Timestamp"),
+                "type": "bool",
+                "default": True,
+                "map_to": "include_timestamp",
+            },
+            "mode": {
+                "name": _("Message Mode"),
+                "type": "choice:string",
+                "values": SLACK_MODES,
+                # mode is detected if not specified
+            },
             "token": {
                 "name": _("Token"),
                 "alias_of": ("access_token", "token_a", "token_b", "token_c"),
@@ -327,16 +346,30 @@ class NotifySlack(NotifyBase):
         targets=None,
         include_image=None,
         include_footer=None,
+        include_timestamp=None,
         use_blocks=None,
+        mode=None,
         **kwargs,
     ):
         """Initialize Slack Object."""
         super().__init__(**kwargs)
 
-        # Setup our mode
-        self.mode = SlackMode.BOT if access_token else SlackMode.WEBHOOK
+        # Store our webhook mode
+        if mode and isinstance(mode, str):
+            self.mode = next(
+                (a for a in SLACK_MODES if a.startswith(mode)), None
+            )
+            if self.mode not in SLACK_MODES:
+                msg = (
+                    f"The Slack mode specified ({mode}) is invalid."
+                )
+                self.logger.warning(msg)
+                raise TypeError(msg)
 
-        if self.mode is SlackMode.WEBHOOK:
+        else:  # Detect
+            self.mode = SlackMode.BOT if access_token else SlackMode.WEBHOOK
+
+        if self.mode in (SlackMode.WEBHOOK, SlackMode.WEBHOOK_GOV):
             self.access_token = None
             self.token_a = validate_regex(
                 token_a, *self.template_tokens["token_a"]["regex"]
@@ -404,7 +437,7 @@ class NotifySlack(NotifyBase):
             # a flag lower to not set the channels
             self.channels.append(
                 None
-                if self.mode is SlackMode.WEBHOOK
+                if self.mode in (SlackMode.WEBHOOK, SlackMode.WEBHOOK_GOV)
                 else self.default_notification_channel
             )
 
@@ -423,6 +456,12 @@ class NotifySlack(NotifyBase):
             self.template_args["footer"]["default"] \
             if include_footer is None else include_footer
 
+        # timestamp inclusion (only applicable if footer also defined
+        self.include_timestamp = \
+            self.template_args["timestamp"]["default"] \
+            if include_timestamp is None \
+            else include_timestamp
+
         return
 
     def send(
@@ -572,10 +611,9 @@ class NotifySlack(NotifyBase):
                     "title": title,
                     "text": body,
                     "color": self.color(notify_type),
-                    # Time
-                    "ts": time(),
                 }],
             }
+
             # Acquire our to-be footer icon if configured to do so
             image_url = (
                 None if not self.include_image else self.image_url(notify_type)
@@ -592,10 +630,13 @@ class NotifySlack(NotifyBase):
                 # Include the footer only if specified to do so
                 payload["attachments"][0]["footer"] = self.app_id
 
+                if self.include_timestamp:
+                    # Timestamp
+                    payload["attachments"][0]["ts"] = time()
         if (
             attach
             and self.attachment_support
-            and self.mode is SlackMode.WEBHOOK
+            and self.mode in (SlackMode.WEBHOOK, SlackMode.WEBHOOK_GOV)
         ):
             # Be friendly; let the user know why they can't send their
             # attachments if using the Webhook mode
@@ -608,6 +649,12 @@ class NotifySlack(NotifyBase):
                 f"/{self.token_b}/{self.token_c}"
             )
 
+        elif self.mode is SlackMode.WEBHOOK_GOV:
+            url = (
+                f"{self.webhook_gov_url}/{self.token_a}"
+                f"/{self.token_b}/{self.token_c}"
+            )
+
         else:  # SlackMode.BOT
             url = self.api_url.format("chat.postMessage")
 
@@ -1115,7 +1162,9 @@ class NotifySlack(NotifyBase):
         params = {
             "image": "yes" if self.include_image else "no",
             "footer": "yes" if self.include_footer else "no",
+            "timestamp": "yes" if self.include_timestamp else "no",
             "blocks": "yes" if self.use_blocks else "no",
+            "mode": self.mode,
         }
 
         # Extend our parameters
@@ -1128,7 +1177,7 @@ class NotifySlack(NotifyBase):
                 botname=NotifySlack.quote(self.user, safe=""),
             )
 
-        if self.mode == SlackMode.WEBHOOK:
+        if self.mode in (SlackMode.WEBHOOK, SlackMode.WEBHOOK_GOV):
             return (
                 "{schema}://{botname}{token_a}/{token_b}/{token_c}/"
                 "{targets}/?{params}".format(
@@ -1233,6 +1282,11 @@ class NotifySlack(NotifyBase):
             parse_bool(results["qsd"].get(
                 "image", NotifySlack.template_args["image"]["default"]))
 
+        results["include_timestamp"] = \
+            parse_bool(results["qsd"].get(
+                "timestamp",
+                NotifySlack.template_args["timestamp"]["default"]))
+
         # Get Payload structure (use blocks?)
         if "blocks" in results["qsd"] and len(results["qsd"]["blocks"]):
             results["use_blocks"] = parse_bool(results["qsd"]["blocks"])
@@ -1242,16 +1296,22 @@ class NotifySlack(NotifyBase):
             parse_bool(results["qsd"].get(
                 "footer", NotifySlack.template_args["footer"]["default"]))
 
+        # Get Mode
+        if "mode" in results["qsd"] and len(results["qsd"]["mode"]):
+            results["mode"] = NotifySlack.unquote(results["qsd"]["mode"])
+
         return results
 
     @staticmethod
     def parse_native_url(url):
         """
-        Support https://hooks.slack.com/services/TOKEN_A/TOKEN_B/TOKEN_C
+        Supports:
+          - https://hooks.slack.com/services/TOKEN_A/TOKEN_B/TOKEN_C
+          - https://hooks.slack-gov.com/services/TOKEN_A/TOKEN_B/TOKEN_C
         """
 
         result = re.match(
-            r"^https?://hooks\.slack\.com/services/"
+            r"^https?://(?P<host>hooks\.slack(?P<gov>-gov)?\.com)/services/"
             r"(?P<token_a>[A-Z0-9]+)/"
             r"(?P<token_b>[A-Z0-9]+)/"
             r"(?P<token_c>[A-Z0-9]+)/?"
@@ -1261,17 +1321,24 @@ class NotifySlack(NotifyBase):
         )
 
         if result:
+            params = (
+                ""
+                if not result.group("params")
+                else result.group("params")
+            )
+
+            if result.group("gov"):
+                # provide gov parameters
+                params = ("?" if not params else "&") + \
+                    f"mode={SlackMode.WEBHOOK_GOV}"
+
             return NotifySlack.parse_url(
                 "{schema}://{token_a}/{token_b}/{token_c}/{params}".format(
                     schema=NotifySlack.secure_protocol,
                     token_a=result.group("token_a"),
                     token_b=result.group("token_b"),
                     token_c=result.group("token_c"),
-                    params=(
-                        ""
-                        if not result.group("params")
-                        else result.group("params")
-                    ),
+                    params=params,
                 )
             )
 
